/* $Id: 542x.c,v 1.6 1998/11/05 02:06:20 marcus Exp $
******************************************************************************

   KGI - Cirrus Logic 542x framebuffer support

   Copyright (C) 1997 Martin Erhardsed	[mee@daimi.aau.dk]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <kgi/maintainers.h>
#define MAINTAINER      Martin_Erhardsen
#define DRIVER_NAME     "CL-GD542x Graphics Driver"
#define DRIVER_REV      "$Revision: 1.6 $"

/*\
|*|     KGI includes
\*/

#include <kgi/module.h>
#include <kgi/kgi_commands.h>
#include "Cirrus/542x.h"

#if 0
/*
**     Accelerated drawing functions
*/

volatile kgiu32 *cirrus_mmio;
/* FIXME: Set this somewhere */

kgi_drawinfo *drwinfo;
/* FIXME: What should this be */

static int pixwidths[KGIGT_LAST] = {0,0,0,0,1,2,2,3,4}; 

static int cirrus_mmio_drawbox(struct kgi_graphic *gr, void *iobuf)
{	
	struct kgi_box *boxp = (struct kgi_box *)iobuf;
	int pixw;

	if (gr->dev.mode.graphtype < KGIGT_8BIT)
		return -E(ACCEL,NOSUP_MODE_MMAP);

	pixw =  pixwidths[gr->dev.mode.graphtype];

	if (boxp->w && boxp->h) {	/* 0 width is not OK! */

		cirrus_mmio[1] = drwinfo->fg_color;
		((kgiu16 *)cirrus_mmio)[4] = boxp->w--;
		((kgiu16 *)cirrus_mmio)[5] = boxp->h--;
		cirrus_mmio[3] = gr->dev.mode.virt.x*pixw;
		cirrus_mmio[4] = gr->dev.mode.virt.x*pixw*boxp->y
			+boxp->x*pixw;
		cirrus_mmio[5] = 0;
		cirrus_mmio[6] = 0x040D00C0 | (pixw<<4);
		((kgiu8 *)cirrus_mmio)[32] = 0x02;
		while (((kgiu8 *)cirrus_mmio)[32]&0x01);
	}

	return(0);
}

static int cirrus_mmio_copybox(struct kgi_graphic *gr,void *iobuf)
{	
	struct kgi_cbox *boxp = (struct kgi_cbox *)iobuf;
	int dir = 0, pixw = 0;

	if (gr->dev.mode.graphtype < KGIGT_8BIT) 
		return -E(ACCEL,NOSUP_MODE_MMAP);

	pixw =  pixwidths[gr->dev.mode.graphtype];

	if (boxp->w&&boxp->h) {		/* 0 width is not OK! */

		/* calculate direction */
		if (boxp->y > boxp->y2) {
			dir = 0;
		} else if (boxp->y < boxp->y2) {
			dir = 1;
		} else if (boxp->y == boxp->y2) {
			dir = (boxp->x < boxp->x2);
		}
		((kgiu16 *)cirrus_mmio)[4] = boxp->w--;
		((kgiu16 *)cirrus_mmio)[5] = boxp->h--;
		cirrus_mmio[3] = gr->dev.mode.virt.x*pixw*0x10001;
		cirrus_mmio[4] = gr->dev.mode.virt.x*pixw*boxp->y2
			+boxp->x2*pixw;
		cirrus_mmio[5] = gr->dev.mode.virt.x*pixw*boxp->y
			+boxp->x*pixw;
		cirrus_mmio[6] = 0x000D0000 | dir;
		((kgiu8 *)cirrus_mmio)[32] = 0x02;
		while (((kgiu8 *)cirrus_mmio)[32]&0x01);		
	}
	return 0;
}
#endif

/*
**	KGI module interface
*/

int kgim_accel_init(struct kgi_card *card, struct kgi_display *dpy)
{
	/* !!! We should really do something here !!! */
	NOTICE(DRIVER_NAME " rev ." DRIVER_REV " loaded.\n");
	return EOK;
}

void kgim_accel_done(struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " removed.\n");
}

int kgim_accel_check_mode (struct kgi_display *dpy, struct kgi_mode *mode,
			    enum kgi_tm_cmd *cmd)
{
	return EOK;
}

void kgim_accel_set_mode (struct kgi_display *dpy, struct kgi_mode *mode)
{
}

int kgim_accel_command (struct kgi_graphic *gr, kgi_uint cmd,kgi_uint arg)
{
	kgi_suggest *sugg;
	const char *name;
	
	switch (cmd) {
	case ACCEL_GETSUGGEST:
		sugg=(kgi_suggest *)(gr->io_buf);
		gr->io_res=gr->io_buf;
		
		if (sugg->id<0 || sugg->id > 1)
			return -E(ACCEL,BOGUS);

		if (sugg->id==1) { /* Generic RAMDAC support */
			if (gr->dev.mode.graphtype>KGIGT_8BIT)
				return -E(ACCEL,BOGUS);
			strcpy(sugg->name,"generic-ramdac");
			sugg->args[0]=0;
			sugg->id=0;
			return 0;
		}
		switch (gr->dev.mode.graphtype) {
		case KGIGT_TEXT16:
			name="generic-text-16";
			break;
		case KGIGT_1BIT:
			name="generic-linear-1";
			break;
		case KGIGT_4BIT:
			name="vendor-IBM-vga-4bpp";
			break;
		case KGIGT_8BIT:
			name="generic-linear-8";
			break;
		case KGIGT_15BIT:
			name="generic-linear-15";
			break;
		case KGIGT_16BIT:
			name="generic-linear-16";
			break;
		case KGIGT_24BIT:
			name="generic-linear-24";
			break;
		case KGIGT_32BIT:
			name="generic-linear-32";
			break;
			
		default:
			return -E(ACCEL,BOGUS);
		}
		strcpy(sugg->name,name);
		sugg->args[0]=0;
		sugg->id=(gr->dev.mode.graphtype<=KGIGT_8BIT);  
		/* Next is RAMDAC.. */
		return 0;
#if 0
	case ACCEL_DRAWBOX:
		if (gr->dev.mode.graphtype>=KGIGT_8BIT) 
			switch(chip_version) 
			{
			case CLGD5436_ID:
			case CLGD5446_ID:
				return cirrus_mmio_drawbox(gr, gr->io_buf);
			case CLGD5434_ID:
			case CLGD5430_ID:
			case CLGD5429_ID:
				return cirrus_mmio_patfill(gr, gr->io_buf);
			case CLGD5428_ID:
			case CLGD5426_ID:
				return cirrus_io_patfill(gr, gr->io_buf);
			}
		return -E(ACCEL,NOSUP_ALWAYS_MMAP);
	case ACCEL_COPYBOX:
		if (gr->dev.mode.graphtype>=KGIGT_8BIT) 
			switch(chip_version) 
			{
			case CLGD5436_ID:
			case CLGD5446_ID:
			case CLGD5434_ID:
			case CLGD5430_ID:
			case CLGD5429_ID:
				return cirrus_mmio_copybox(gr, gr->io_buf);
			case CLGD5428_ID:
			case CLGD5426_ID:
				return cirrus_io_copybox(gr, gr->io_buf);
			}
		return -E(ACCEL,NOSUP_ALWAYS_MMAP);
#endif
	default:
		break;
	}
			
	return -E(ACCEL,NOSUP_ALWAYS_MMAP);
}
