/* $Id: gffd.h,v 1.3 2000/03/31 05:20:12 ajapted Exp $
******************************************************************************

   LibGFFD API header file

   Copyright (C) 2000   Andrew Apted    [andrew.apted@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _GFFD_GFFD_H
#define _GFFD_GFFD_H

#ifndef _INTERNAL_LIBGFFD
/* Opaque pointer types. This little magic gains us some type checking. */
struct gffd_h_dummy_ctx { char dummy_ctx; };
typedef struct gffd_h_dummy_ctx *gffd_context_t;
#endif


/* Error returns */

#define GFFD_E_ERROR            -400   /* catch-all */
#define GFFD_E_INVALID          -401   /* invalid parameters */
#define GFFD_E_BAD_PLACE        -402   /* function used out of place */
#define GFFD_E_BAD_OPT          -403   /* unknown option */
#define GFFD_E_NO_MEM           -404   /* out of memory */


/* Options */

/* ... none yet */


/*
******************************************************************************
 libGFFD function definitions
******************************************************************************
*/

__BEGIN_DECLS

/* Enter and leave the library
 */
int gffdInit(void);
int gffdExit(void);


/* Obtain and release a context for performing matches
 */
gffd_context_t gffdNewContext(void);
int gffdFreeContext(gffd_context_t ctx);


/* Option management
 */
int gffdSetOption(gffd_context_t ctx, int option, int value);
int gffdGetOption(gffd_context_t ctx, int option);


/* Begin and end a matching sequence
 */
int gffdBegin(gffd_context_t ctx);
int gffdEnd(gffd_context_t ctx);


/* Matching operations
 */
int gffdMatchName(gffd_context_t ctx, const char *name);
int gffdMatchData(gffd_context_t ctx, const unsigned char *data,
    int length);

int gffdPruneScore(gffd_context_t ctx, int score);
int gffdPruneClass(gffd_context_t ctx, const char *class);
int gffdPruneType(gffd_context_t ctx, const char *type);
int gffdPruneMIME(gffd_context_t ctx, const char *MIME);


/* Query operations
 */
int gffdGetCount(gffd_context_t ctx);
int gffdGetMatchScore(gffd_context_t ctx, int index);

const char *gffdGetMatchClass(gffd_context_t ctx, int index);
const char *gffdGetMatchType(gffd_context_t ctx, int index);
const char *gffdGetMatchMIME(gffd_context_t ctx, int index);

__END_DECLS

#endif /* _GFFD_GFFD_H */
